%
% Institute for Signal Processing (University of Luebeck, Germany)
% Copyright (c) 2010 by Florian Mueller
%
% Permission to use, copy, modify, and distribute this software without
% fee is hereby granted FOR RESEARCH/EDUCATION PURPOSES only, provided
% that this copyright notice appears in all copies and in all supporting
% documentation, and that the software is not redistributed for any
% fee (except for a nominal shipping charge).
%
% For any other uses of this software, in original or modified form,
% including but not limited to consulting, production or distribution
% in whole or in part, specific prior permission must be obtained
% from the author.
% Signal processing methods and algorithms implemented by this
% software may be claimed by patents owned by others.
%
% The author makes no representation about the suitability of this
% software for any purpose. It is provided "as is" without warranty
% of any kind, either expressed or implied.
% Beware of the bugs.
%
%     Revision history
%
%     Ver     Date         Description
%     -------------------------------------------------------------------
%     0.9     01-05-2010   basic version 
%     0.9.2   01-07-2010   Recoded in C++ with OOP approach
%


%% Exemplary script for showing the usage of computeIIF

addpath('demofiles');

fprintf(['It is assumed that the binary computeIIF is compiled and located.\n' ...
         'in the subfolder ''bin'' relative to the current folder\n' ...
         'Press key to continue.\n']);
pause;

% check for binary
systemtype = computer('arch');
if strcmp(systemtype,'win32') || strcmp(systemtype,'win64')
    binname = '.\bin\computeIIF.exe';
    command = [binname ' demofiles\iifset30.cfg demofiles\demolist.wavin demofiles\demolist.htkout'];
    command2 = [binname ' -g -n 110 -f 40 -v 8000 demofiles\demoiifset.cfg demofiles\demolist.wavin demofiles\demolist.htkout'];
elseif strcmp(systemtype,'glnx86') || strcmp(systemtype,'glnxa64')
    binname = './bin/computeIIF';
    command = [binname ' demofiles/iifset30.cfg demofiles/demolist.wavin demofiles/demolist.htkout'];
    command2 = [binname ' -g -n 110 -f 40 -v 8000 demofiles/demoiifset.cfg demofiles/demolist.wavin demofiles/demolist.htkout'];
end
if ~exist(binname,'file')
    error('The binary ''%s'' cannot be found.',binname );
end
    
%% Now, we are going to make some demo calls

fprintf(['Now, we are going to make some demo calls.\n' ...
         'First, we call computeIIF without any parameters ...\n']);

system(binname);

fprintf(['\nAs you can see, help information is displayed that explains\n' ...
         'all parameters of the program.\n' ...
         'Now we will compute 30 IIF features of order 1 for two given\n' ...
         'speech signals in int16 raw format. Note that also IIF sets\n' ...
         'with 20 and 10 elements are located in the demofiles folder.\n' ...
         'Press key to continue.\n']);
pause;

if ~system(command)==0
    error(['Error while executing ''' command '''']);
end

% We use the readhtk method from Mike Brookes VOICEBOX toolbox 
% (http:%www.ee.ic.ac.uk/hp/staff/dmb/voicebox/voicebox.html) here.
A = readhtk('demofiles/signal-demo-iif.htk');
figure, imagesc(A'), axis xy,xlabel('Frame [#]'), ylabel('IIF index');

fprintf(['\ncomputeIIF can also be used for just computing a time-frequency representation\n' ...
         'with a Gammatone filter bank. The following example computed such a representation\n' ...
         'with a minimal frequency band located at 40Hz, the maximum frequency band located at 8kHz,\n' ...
         'each equally spaces on the ERB scale with an overall number of subbands of 110.\n' ...
         'Press key to continue\n']);
pause;

if ~system(command2)==0
    error(['Error while executing ''' command2 '''']);
end

A = readhtk('demofiles/signal-demo-iif.htk');
figure, imagesc(A'), axis xy, axis image, xlabel('Frame [#]'), ylabel('Subband index');

fprintf('This is the end of the demo.\n');
