//
// Institute for Signal Processing (University of Luebeck, Germany)
// Copyright (c) 2010 by Florian Mueller
//
// Permission to use, copy, modify, and distribute this software without
// fee is hereby granted FOR RESEARCH/EDUCATION PURPOSES only, provided
// that this copyright notice appears in all copies and in all supporting
// documentation, and that the software is not redistributed for any
// fee (except for a nominal shipping charge).
//
// For any other uses of this software, in original or modified form,
// including but not limited to consulting, production or distribution
// in whole or in part, specific prior permission must be obtained
// from the author.
// Signal processing methods and algorithms implemented by this
// software may be claimed by patents owned by others.
//
// The author makes no representation about the suitability of this
// software for any purpose. It is provided "as is" without warranty
// of any kind, either expressed or implied.
// Beware of the bugs.
//
//     Revision history
//
//     Ver     Date         Description
//     -------------------------------------------------------------------
//     0.9     01-05-2010   basic version 
//     0.9.2   01-07-2010   Recoded in C++ with OOP approach
//

#include "ConsoleHelpers.h"
#include <vector>

using namespace std;


/***************
 * Constructor *
 ***************/
ConsoleHelpers::ConsoleHelpers(void)
{
    useCF = false;               // flag, if set: use given center frequencys
    compression=0.1;            // compression factor
    samplerate=16000;           // samplerate [Hz]
    freqmin=40;                 // minimum frequency [Hz]
    freqmax=samplerate/2;       // maximum frequency [Hz]
    nfilter=90;                 // number of filters
    frameshift=0.01;            // frame shift [ms]
    framesize=0.02;             // frame size [ms]
    width=1;                    // scaling factor for relative-bandwidth
    calcGFB=true;
    calcIIF=true;
    preemphasize=1;
    meannorm=true;
    stdnorm=false;
    inputfile="";
    outputfile="";
    CFfile="";
    configfile="";
    DOZEROPAD=false;

}
/*************************************
 * Check for by parameter set values *
 *************************************/

void ConsoleHelpers::readConsoleParameters(int argc, char **argv)
{

    /******************************
     * starting help-informations *
     ******************************/
    if (argc==1)
    {
        help();
        exit(0);
    }
    if ((argc==1)|(strcmp(argv[1], "help")==0)|(strcmp(argv[1], "-h")==0)|(strcmp(argv[1], "-help")==0)|(argc<3))
    {
        help();
        exit(0);
    }

    int k;
    for (k=3; k<argc;k++)
    {
        // for gammatone filterbank
        if ((strcmp(argv[k], "-r")==0)||(strcmp(argv[k], "--samplerate")==0))
            samplerate=atof(argv[k+1]);
        if ((strcmp(argv[k], "-f")==0)||(strcmp(argv[k], "--minimum-frequency")==0))
            freqmin=atof(argv[k+1]);
        if ((strcmp(argv[k], "-v")==0)||(strcmp(argv[k], "--maximum-frequency")==0))
            freqmax=atof(argv[k+1]);
        if ((strcmp(argv[k], "-n")==0)||(strcmp(argv[k], "--number-filters")==0))
            nfilter=atof(argv[k+1]);
        if ((strcmp(argv[k], "-m")==0)||(strcmp(argv[k], "--no-mean")==0))
            meannorm=false;
        if ((strcmp(argv[k], "-t")==0)||(strcmp(argv[k], "--stdDeviation")==0))
            stdnorm=true;
        if ((strcmp(argv[k], "-h")==0)||(strcmp(argv[k], "--frame-shift")==0))
            frameshift=atof(argv[k+1]);
        if ((strcmp(argv[k], "-e")==0)||(strcmp(argv[k], "--frame-size")==0))
            framesize=atof(argv[k+1]);
        if ((strcmp(argv[k], "-w")==0)||(strcmp(argv[k], "--relative-bandwidth")==0))
            width=atof(argv[k+1]);
        if ((strcmp(argv[k], "-p")==0)||(strcmp(argv[k], "--no-preemphasize")==0))
            preemphasize=true;
        if ((strcmp(argv[k], "-i")==0)||(strcmp(argv[k], "--iif-only")==0))
            calcGFB=false;
        if ((strcmp(argv[k], "-g")==0)||(strcmp(argv[k], "--filterbank-only")==0))
            calcIIF=false;
        if ((strcmp(argv[k], "-z")==0)||(strcmp(argv[k], "--zero-pad")==0))
            DOZEROPAD=true;
        if ((strcmp(argv[k], "-c")==0)||(strcmp(argv[k], "--compression-factor")==0))
        {
            CFfile=argv[k+1];
        }
    }
    if (calcIIF)
    {
         if (argc<4)
         {
                help();
                exit(0);
         }
        configfile=argv[argc-3];
        inputfile= argv[argc-2];
        outputfile = argv[argc-1];
    }
    else
    {
        inputfile= argv[argc-2];
        outputfile = argv[argc-1];
    }


}

void ConsoleHelpers::help()
{
    printf("invintfeatures - help\n");
    printf("\nCall: ComputeIIF [OPTIONS]... [CONFIGFILE] INPUTFILELIST OUTPUTFILELIST\n\n");
    printf("set CONFIGFILE or set (-i|--iif-only)\n");
    printf("optional parameters are available in short and long form\n\n");
    printf("short,\tlong form\t\targument\n");
    printf("------------------------------------------------------------------------------\n");
    printf("-z  | --zero-pad\t\t\tfor pad with zeros'\n\t\t\t\tdefault: pad with boundary\n\n");
    printf("-g  | --filterbank-only\t\tcalculate only gammatone values without iifs\n\n");
    printf("-i  | --iif-only\t\t\tcalculate only iif.\n\t\t\t\tIMPORTANT: Filelist must contain htk-files\n\n");
    printf("-r  | --samplerate\t\tfollowed by INT to set samplerate [Hz]\n\t\t\t\tdefault: 16000\n\n");
    printf("-f  | --minimum-frequency\tfollowed by DOUBLE to set min frequency [Hz]\n\t\t\t\tdefault: 40\n\n");
    printf("-v  | --maximum-frequency\tfollowed by DOUBLE to set max frequency [Hz]\n\t\t\t\tdefault: samplerate/2\n\n");
    printf("-n  | --number-filters\t\tfollowed by INT to set number of filter\n\t\t\t\tdefault: 90\n\n");
    printf("-h  | --frame-shift\t\tfollowed by DOUBLE to set frame shift [s]\n\t\t\t\tdefault: 0.01\n\n");
    printf("-e  | --frame-size\t\tfollowed by DOUBLE to set frame length [s]\n\t\t\t\tdefault: 0.02\n\n");
    printf("-w  | --relative-bandwidth\tfollowed by DOUBLE to set scaling factor for\n\t\t\t\tbandwidth of gammatone filters\n\t\t\t\tdefault: 1\n\n");
    printf("-p  | --preemphasize\t\tfollowed by DOUBLE to set preemphasize\n\t\t\t\tdefault: 1 (= no preemphasize)\n\n");
    printf("-cf | --center-frequency\t\tfollowed by CHAR to set filename\n\t\t\t\tof own center frequencys\n\n");
    printf("-c  | --compression-factor\tfollowed by DOUBLE to set compression-factor\n\t\t\t\tdefault: 0.1\n\n");
    printf("-m  | --no-mean\t\t\tset no mean norm\n\n");
    printf("-t  | --stdDeviation\t\tset standard deviation norm.\n\n");
}

ConsoleHelpers::~ConsoleHelpers()
{
    //dtor

}

