//
// Institute for Signal Processing (University of Luebeck, Germany)
// Copyright (c) 2010 by Florian Mueller
//
// Permission to use, copy, modify, and distribute this software without
// fee is hereby granted FOR RESEARCH/EDUCATION PURPOSES only, provided
// that this copyright notice appears in all copies and in all supporting
// documentation, and that the software is not redistributed for any
// fee (except for a nominal shipping charge).
//
// For any other uses of this software, in original or modified form,
// including but not limited to consulting, production or distribution
// in whole or in part, specific prior permission must be obtained
// from the author.
// Signal processing methods and algorithms implemented by this
// software may be claimed by patents owned by others.
//
// The author makes no representation about the suitability of this
// software for any purpose. It is provided "as is" without warranty
// of any kind, either expressed or implied.
// Beware of the bugs.
//
//     Revision history
//
//     Ver     Date         Description
//     -------------------------------------------------------------------
//     0.9     01-05-2010   basic version 
//     0.9.2   01-07-2010   Recoded in C++ with OOP approach
//

#ifndef FILELIST_H
#define FILELIST_H
#include <vector>
#include <iostream>
#include <sstream>
#include <fstream>
#include <string.h>
#include"array2d.h"
#include "ConsoleHelpers.h"
#include <complex>

using namespace std;

typedef vector<string> sVec;
typedef complex<double> cDouble;

class Filelist
{
public:
    Filelist();
    virtual ~Filelist();
    Filelist(string);
    dVec getSignal();
    int length();
    string getIterator();
    void getNext();
    void writehtk(Array2d<double>, double);
    Array2d<double> readhtk(double);
private:
    dVec getSignal(int);
    unsigned int currentIndex;
    sVec sData;
    string filename;
    void writehtk(string, Array2d<double>, double);
    /*++++++++++++Endian swap methods+++++++++++++*/
    void FFT(int, long, cDouble);
    bool LittleEndian;
    void isBigEndian();
    float endian_swap(float);
    double endian_swap(double);
    unsigned int endian_swap(unsigned int &);
    short endian_swap( short);
    unsigned short endian_swap(unsigned short& x);
    Array2d<double> readhtk(string filename, double frameshift);
};

#endif // FILELIST_H
