//
// Institute for Signal Processing (University of Luebeck, Germany)
// Copyright (c) 2010 by Florian Mueller
//
// Permission to use, copy, modify, and distribute this software without
// fee is hereby granted FOR RESEARCH/EDUCATION PURPOSES only, provided
// that this copyright notice appears in all copies and in all supporting
// documentation, and that the software is not redistributed for any
// fee (except for a nominal shipping charge).
//
// For any other uses of this software, in original or modified form,
// including but not limited to consulting, production or distribution
// in whole or in part, specific prior permission must be obtained
// from the author.
// Signal processing methods and algorithms implemented by this
// software may be claimed by patents owned by others.
//
// The author makes no representation about the suitability of this
// software for any purpose. It is provided "as is" without warranty
// of any kind, either expressed or implied.
// Beware of the bugs.
//
//     Revision history
//
//     Ver     Date         Description
//     -------------------------------------------------------------------
//     0.9     01-05-2010   basic version 
//     0.9.2   01-07-2010   Recoded in C++ with OOP approach
//

#ifndef GAMMATONEFB_H
#define GAMMATONEFB_H
#include <string.h>
#include <iostream>
#include <vector>
#include <iostream>
#include <sstream>
#include <fstream>
#include <cstdlib>
#include"array2d.h"
#include <cmath>
#include <complex>
#include <string.h>

typedef std::vector<double> dVec;
typedef std::complex<double> cDouble;
typedef std::vector<cDouble> cVec;


class GammatoneFB
{
public:
    GammatoneFB();
    virtual ~GammatoneFB();
    Array2d<double> computeGTFOutput(dVec, double, double, double, double, double, double, double, double,double, bool, bool, std::string);
protected:
private:
    Array2d<double> makeFrames(double, double,double,dVec);
    bool FFT(cVec &, const unsigned int);
    static void Rearrange(cVec &, const unsigned int);
    static void Perform(cVec & Data, const unsigned int N, const bool Inverse = false);
    dVec rmsNorm(dVec);
    Array2d<double> removeDV(Array2d<double>);
    Array2d<double> hamming(Array2d<double>);
    Array2d<double> preemphasize(double, Array2d<double>);
    cVec fillCframes(Array2d<double>,int index,  long int );
    long int nextPower2(int);
    Array2d<double>  doFFT(Array2d<double>);
    dVec readCF(std::string);
    dVec calcCenterFrequencies (double, double, double, dVec);
    Array2d<double> calcWeights(int, double, double, double, double, double, dVec, bool, bool);
    Array2d<double> sMult(Array2d<double>, Array2d<double>);
    Array2d<double> transAndFac(Array2d<double>, double);

};

#endif // GAMMATONEFB_H
