//
// Institute for Signal Processing (University of Luebeck, Germany)
// Copyright (c) 2010 by Florian Mueller
//
// Permission to use, copy, modify, and distribute this software without
// fee is hereby granted FOR RESEARCH/EDUCATION PURPOSES only, provided
// that this copyright notice appears in all copies and in all supporting
// documentation, and that the software is not redistributed for any
// fee (except for a nominal shipping charge).
//
// For any other uses of this software, in original or modified form,
// including but not limited to consulting, production or distribution
// in whole or in part, specific prior permission must be obtained
// from the author.
// Signal processing methods and algorithms implemented by this
// software may be claimed by patents owned by others.
//
// The author makes no representation about the suitability of this
// software for any purpose. It is provided "as is" without warranty
// of any kind, either expressed or implied.
// Beware of the bugs.
//
//     Revision history
//
//     Ver     Date         Description
//     -------------------------------------------------------------------
//     0.9     01-05-2010   basic version 
//     0.9.2   01-07-2010   Recoded in C++ with OOP approach
//

#include "IIFcomputation.h"

using namespace std;


IIFcomputation::IIFcomputation()
{
    maxConfigNumber=0;
    maxConfigLines=0;
    ConfigLines=0;
}



IIFcomputation::~IIFcomputation()
{
    validConfigLines.clear();
    NumbersInLine.clear();
    maxConfigNumber=0;
    maxConfigLines=0;
    ConfigLines=0;
}

/********************************************************************************
 *                      getConfigSize                                           *
 * set maxConfigLines,  maxConfigNumber, validConfigLines and  NumbersInLine    *
 *                                                                              *
 * INPUT:                       string              filename                    *
 * OUTPUT*:                     int                 maxConfigLines              *
 *                              int                 ConfigLines                 *
 *                              int                 maxConfigNumber             *
 *                              iVec                NumbersInLine               *
 *                              iVec                validConfigLines            *
 *                                                                              *
 *  *(saved in class)                                                           *
 *                                                                              *
 *  maxConfigLines:     number of lines in file                                 *
 *  ConfigLines:        number of lines with configuration values               *
 *  maxConfigNumber:    max. values in line                                     *
 *  NumbersInLine:      size of configuations line                              *
 *  validConfigLines:   contains indeces from lines with configuration values   *
 *                                                                              *
 * June 2010  Peter-Christian Quint, pcq@gmx.de                                 *
 ********************************************************************************/

void IIFcomputation::getConfigSize(string filename)
{
    ifstream file;
    file.open(filename.c_str(), ios::in);
    iVec positions;
    int zeichen=0;
    bool flag=true;
    char tempChar;
    complex<double> temp;
    maxConfigLines=0;
    if (file.good())
    {
        while (!file.eof()) // Zeilenweise
        {
            zeichen=0;
            tempChar=file.get();
            if (tempChar=='#')
            {
                while ((tempChar!='\n')&&(!file.eof()))
                {
                    tempChar=file.get();    //do nothing
                }
            }
            else
            {
                while ((tempChar!='\n')&&(!file.eof())) //character by character
                {
                    if (tempChar==' ')   //if whitespace, next number
                    {
                        flag=true;      // if flag set: first digit of number
                    }
                    else                 //if no whitespace save value
                    {
                        // we have a negativ value, the first digit was '-', but the next digis are numbers

                        if (tempChar!='-')
                        {
                            if (flag)           // first letter of number
                            {
                                zeichen++;
                                flag=false;
                            }
                        }
                    }
                    tempChar=file.get();    // get next char
                }
                NumbersInLine.push_back(zeichen);
                validConfigLines.push_back(maxConfigLines);
                if (maxConfigNumber<zeichen)
                {
                    maxConfigNumber=zeichen;
                }
                zeichen=0;
                flag=true;
            }
            maxConfigLines++;
        }
    }
    else
    {
        printf("ERROR: Cannot open configuration file for reading %s for reading. \n",filename.c_str());
        file.close();
        exit(12);
    }
    NumbersInLine.pop_back();// delite last line (while-loop makes one emtpy line to much)
    validConfigLines.pop_back();
    maxConfigLines--;
    ConfigLines=validConfigLines.size();

    file.close();
}

/****************************************************************
 *                      readConfigFile                          *
 * read IIF configuration file (filename) and save              *
 * in matrix (configData).                                      *
 *                                                              *
 * INPUT:                       string              filename    *
 * OUTPUT: (saved in class)     Array2d<int>        configData  *
 *                                                              *
 * June 2010  Peter-Christian Quint, pcq@gmx.de                 *
 ****************************************************************/

void IIFcomputation::readConfigFile(string filename)
{
    int iifctr=0;
    int i,j,k;
    char tempChar;
    bool negFlag=false;
    bool flag=true;
    iifctr=0;
    char waste[1024];
    getConfigSize(filename); // set maxConfigLines,  maxConfigNumber, validConfigLines and  NumbersInLine;
    configData.setSize(ConfigLines,maxConfigNumber,0);
    ifstream file;
    file.open(filename.c_str(), ios::in);
    if (file.good())
    {
        j=0;
        for (i=0;i<maxConfigLines;i++)
        {
            k=0;
            if (i!=validConfigLines[j])              // unused comment line
            {
                file.getline(waste,sizeof(waste));
            }
            else                                    // wanted data
            {
                tempChar=file.get();
                while ((tempChar!='\n')&&(!file.eof())) //character by character
                {
                    if (tempChar==' ')   //if whitespace, next number
                    {
                        k++;
                        flag=true;      // if flag set: first digit of number
                        negFlag=false;  // if negFlag set: the first digit of number was '-'
                    }
                    else
                    {
                        if (tempChar=='-')      // we have a negativ value, the first digit was '-', but the next digis are numbers
                        {
                            negFlag=true;
                        }
                        else
                        {
                            if (flag)           // first letter of number
                            {
                                if (negFlag)    // negativ value
                                {
                                    configData[j][k]=(-1*(atoi(&tempChar)));
                                }
                                else            // positiv value
                                {
                                    configData[j][k]=(atoi(&tempChar));
                                }
                                negFlag=false;
                                flag=false;
                            }
                            else                //if not the first digit
                            {
                                configData[j][k]=configData[j][k]*10+atoi(&tempChar); //change last element of list
                            }
                        }
                    }
                    tempChar=file.get();

                }
                j++;
            }
        }
    }
    /*
        for (i=0;i<configData.getZeilen();i++)
        {
            for (k=0;k<configData.getSpalten();k++)
            {
                cout<<configData[i][k]<<"  ";
             }
            cout<<endl;
        }
    */
}

/****************************************************************************************
 *                                      computeIIFs                                     *
 * compute invariant integration features (IIFs)                                        *
 *                                                                                      *
 * INPUT:   Array2d<double>             htkmat          HTK-data                        *
 *          bool                        DOZEROPAD       true for pad with zeros         *
 *          bool                        meannorm        true for do meannorm            *
 *          bool                        stdnorm         true for do standardnorm        *
 * OUTPUT:  Array2d<double>             iifmat          result matrix                   *
 *                                                                                      *
 * June 2010  Peter-Christian Quint, pcq@gmx.de                                         *
 ****************************************************************************************/
Array2d<double> IIFcomputation::computeIIFs(Array2d<double> htkmat, bool DOZEROPAD, bool meannorm, bool stdnorm)
{
    int htkDim1=htkmat.getZeilen();
    int htkDim2=htkmat.getSpalten();
    int iffdefDim=configData.getZeilen();
    int iffdefDim2=configData.getSpalten();

    Array2d<double> iifmat(htkDim1,iffdefDim);
    Array2d<double> tempfeats;
    Array2d<double> tempmat;
    Array2d<double> featmat_padded;


    double monomialOrder; //for normalize features
    int maxel; //max monomia
    int minel; //min monomia
    double offset;
    int numcomps;
    int winlength;
    dVec winindices;
    unsigned int unsi, unsi2, widx;  //unsigned run variable
    int i, midx, cidx, front, back;
    for (midx=0;midx<iffdefDim;midx++) //Schleife 1
    {
        monomialOrder=0;
        for (i=2;i<NumbersInLine[midx];i+=3)
        {
            monomialOrder+=configData[midx][i];
        }
        monomialOrder=1/monomialOrder;
        winindices.clear();
        iffdefDim2=NumbersInLine[midx];
        winlength=(int) configData[midx][0];
        //get minel and maxel
        maxel=-1000;
        minel=1000;
        numcomps=0;
        for (i=1;i<iffdefDim2;i+=3)
        {
            if (maxel<configData[midx][i])
                maxel=(int) configData[midx][i];

            if (minel>configData[midx][i])
                minel=(int) configData[midx][i];
            numcomps++;
        }
        for (i=winlength*(-1);i<=winlength;i++) //winindices = -winlength:winlength;
        {

            if (DOZEROPAD)
            {
                if (((i+minel<1)or(i+maxel>htkDim2))==false)
                    winindices.push_back(i);

            }
            else
            {
                winindices.push_back(i-min(minel-winlength-1,0));
            }
        }
        if (DOZEROPAD)
            featmat_padded=htkmat;
        else
        {
            front=max(winlength-minel+1,0);
            back=maxel+winlength-htkmat.getSpalten();
            back=max(back,0);
            featmat_padded.resize(htkmat.getZeilen(), htkmat.getSpalten()+front+back);
            for (unsi=0;unsi< htkmat.getZeilen();unsi++)
            {
                for (unsi2=0;unsi2<(unsigned int)front;unsi2++)      //don't worry: front can not be negativ, see 6 lines before
                {
                    featmat_padded[unsi][unsi2]=htkmat[unsi][0];
                }

                for (unsi2=0;unsi2<htkmat.getSpalten();unsi2++)
                {
                    featmat_padded[unsi][unsi2+front]=htkmat[unsi][unsi2];
                }

                for (unsi2=htkmat.getSpalten()+front;unsi2<htkmat.getSpalten()+front+back;unsi2++)
                {
                    featmat_padded[unsi][unsi2]=htkmat[unsi][htkmat.getSpalten()-1];
                }
            }
        }

        tempmat.resize(htkDim1, winindices.size());
        // compute the values of each summand
        for (cidx=0; cidx<numcomps;cidx++)
        {
            offset=configData[midx][3+(cidx*3)]; // take care for (time) offset;
            tempfeats.setSize(htkDim1,winindices.size()+abs(offset),0.0);


            if (offset<=0)
            {
                for (i=0;i<abs(offset);i++)
                {
                    for (widx=0;widx<winindices.size();widx++)
                    {
                        tempfeats[i][widx]=featmat_padded[0][configData[midx][(1+cidx*3)]+(int) winindices[widx]-1];
                    }

                }
                for (i=0;i<htkDim1+offset;i++)
                {
                    for (widx=0;widx<winindices.size();widx++)
                    {
                        tempfeats[i+abs(offset)][widx]=featmat_padded[i][configData[midx][1+(cidx*3)]+(int) winindices[widx]-1];

                    }
                }

            }
            else // offset>0
            {

                for (i=0;i<htkDim1-offset;i++)
                {
                    for (widx=0;widx<winindices.size();widx++)
                    {
                        tempfeats[i][widx]=featmat_padded[i+offset][configData[midx][1+(cidx*3)]+(int) winindices[widx]-1];

                    }
                }

                for (i=htkDim1-offset;i<htkDim1;i++)
                {
                    for (widx=0;widx<winindices.size();widx++)
                    {
                        tempfeats[i][widx]=featmat_padded[htkDim1-1][configData[midx][1+(cidx*3)]+(int) winindices[widx]-1];
                    }
                }
            }

            if (cidx==0)
            {
                for (unsi=0;unsi<tempmat.getZeilen();unsi++)
                {
                    for (unsi2=0;unsi2<tempmat.getSpalten();unsi2++)
                    {
                        tempmat[unsi][unsi2]=pow(tempfeats[unsi][unsi2],configData[midx][2+(cidx*3)]);

                    }
                }
            }
            else
            {
                for (unsi=0;unsi<tempmat.getZeilen();unsi++)
                {
                    for (unsi2=0;unsi2<tempmat.getSpalten();unsi2++)
                    {
                        tempmat[unsi][unsi2]=tempmat[unsi][unsi2]*pow(tempfeats[unsi][unsi2],configData[midx][2+(cidx*3)]);
                    }
                }
            }
        }
        // finally, compute the mean value of the monomials
        for (i=0;i<htkDim1;i++)
        {
            iifmat[i][midx]=0;
            for (unsi=0;unsi<winindices.size();unsi++)
            {
                iifmat[i][midx]+= pow(tempmat[i][unsi],monomialOrder); //normalize features w.r.t. the monomial order
            }
            iifmat[i][midx]=iifmat[i][midx]/winindices.size();
        }
    } // Ende Schleife 1
    winindices.clear();
    iifmat=subductMean(iifmat, meannorm, stdnorm);
    return iifmat;
}

/********************************************************************************
 *                              subductMean                                     *
 * 	Mean normalization                                                          *
 *                                                                              *
 * INPUT:   Array2d<double>     val             values for normalization        *
 *          bool                meannorm        true for do meannorm            *
 *          bool                stdnorm         true for do standardnorm        *
 *                                                                              *
 * OUTPUT:  Array2d<double>     val             result matrix                   *
 *                                                                              *
 * June 2010  Peter-Christian Quint, pcq@gmx.de                                 *
 ********************************************************************************/
Array2d<double> IIFcomputation::subductMean(Array2d<double> val, bool meannorm, bool stdnorm)
{
    double stdDev;
    if (meannorm)
    {
        double mean;
        unsigned int i,j;
        for (i=0; i<val.getSpalten();i++)
        {
            mean=0;
            for (j=0;j<val.getZeilen();j++)
            {
                mean+=val[j][i];
            }
            mean=mean/val.getZeilen();
            // norm with mean and standard deviation
            if (stdnorm)
            {
                stdDev=0;

                for (j=0;j<val.getZeilen();j++)
                {
                    stdDev+=pow(val[j][i]-mean,2);
                }
                stdDev=pow((stdDev/val.getZeilen()),0.5);

                for (j=0;j<val.getZeilen();j++)
                {
                    val[j][i]-=mean;
                    val[j][i]=val[j][i]/stdDev;
                }

            }
            // norm with mean only
            else
            {
                for (j=0;j<val.getZeilen();j++)
                {
                    val[j][i]-=mean;
                }
            }
        }
    }
    return val;
}

