//
// Institute for Signal Processing (University of Luebeck, Germany)
// Copyright (c) 2010 by Florian Mueller
//
// Permission to use, copy, modify, and distribute this software without
// fee is hereby granted FOR RESEARCH/EDUCATION PURPOSES only, provided
// that this copyright notice appears in all copies and in all supporting
// documentation, and that the software is not redistributed for any
// fee (except for a nominal shipping charge).
//
// For any other uses of this software, in original or modified form,
// including but not limited to consulting, production or distribution
// in whole or in part, specific prior permission must be obtained
// from the author.
// Signal processing methods and algorithms implemented by this
// software may be claimed by patents owned by others.
//
// The author makes no representation about the suitability of this
// software for any purpose. It is provided "as is" without warranty
// of any kind, either expressed or implied.
// Beware of the bugs.
//
//     Revision history
//
//     Ver     Date         Description
//     -------------------------------------------------------------------
//     0.9     01-05-2010   basic version 
//     0.9.2   01-07-2010   Recoded in C++ with OOP approach
//

#include <iostream>
#include "GammatoneFB.h"
#include "ConsoleHelpers.h"
#include <string.h>
#include <iostream>
#include <vector>
#include <iostream>
#include <sstream>
#include <fstream>
#include <cmath>
#include <cstdlib>
#include <cmath>
#include "array2d.h"
#include "Filelist.h"
#include "IIFcomputation.h"

using namespace std;
typedef vector<double> dVec;
typedef complex<double> cDouble;
typedef vector<cDouble> cVec;


ConsoleHelpers cHelpers;
IIFcomputation cIIF;
GammatoneFB gammafb;

bool DEBUGMODE=true;

int main(int argc, char **argv)
{
    Array2d<double> gtfresult;
    Array2d<double> iifmat;
    dVec signal;
    /** Read Paramters **/
    cHelpers.readConsoleParameters(argc, argv);
    /** Read input and output filelist **/
    Filelist flInput(cHelpers.inputfile);
    Filelist flOutput(cHelpers.outputfile);
    if (cHelpers.calcIIF)
    {
        /** Read IIF configuation file **/
    	cIIF.readConfigFile(cHelpers.configfile);
    }
	
    double counter=0;
    for ( int fidx=0; fidx<flInput.length(); fidx++ )
    {
       

        /** get next input and output filename **/
        string fileinput = flInput.getIterator();
        string fileoutput = flOutput.getIterator();
        printf("%.2f%% (%d: %s -> %s)\r",counter, fidx+1,fileinput.c_str(), fileoutput.c_str());
	fflush(stdout);
        counter+=100.0/(double)flInput.length();
        if (cHelpers.calcGFB)
        {
            /** read raw wave signal **/
            signal = flInput.getSignal();
            /** compute gammtone filterbank **/
            gtfresult =  gammafb.computeGTFOutput(signal,cHelpers.samplerate, cHelpers.framesize, cHelpers.frameshift, cHelpers.width, cHelpers.preemphasize, cHelpers.nfilter, cHelpers.freqmin, cHelpers.freqmax, cHelpers.compression, cHelpers.meannorm, cHelpers.stdnorm,cHelpers.CFfile);
        }
        else
        {
            /** read gammatone filterbank values from file **/
            gtfresult = flInput.readhtk(cHelpers.frameshift);
        }

        if (cHelpers.calcIIF)
        {
            /** compute invariant integration features (IIFs) **/
            iifmat = cIIF.computeIIFs(gtfresult, cHelpers.DOZEROPAD, cHelpers.meannorm, cHelpers.stdnorm);
            /** write IIFs in file **/
            flOutput.writehtk(iifmat, cHelpers.frameshift);
        }
        else
        {
            /** write gammtone filterbank values to file **/
            flOutput.writehtk(gtfresult, cHelpers.frameshift);
        }
        flInput.getNext();
        flOutput.getNext();
        signal.clear();
    }
    printf("100%% -> Finished (%d files)\r",flInput.length());
    return 0;
}
